"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ow_1 = __importDefault(require("ow"));
var logFactorialTable = [
    0.0,
    0.0,
    0.69314718055994529,
    1.791759469228055,
    3.1780538303479458,
    4.7874917427820458,
    6.5792512120101012,
    8.5251613610654147,
    10.604602902745251,
    12.801827480081469
];
var logFactorial = function (k) {
    return logFactorialTable[k];
};
var logSqrt2PI = 0.91893853320467267;
exports.default = (function (random, lambda) {
    if (lambda === void 0) { lambda = 1; }
    ow_1.default(lambda, ow_1.default.number.positive);
    if (lambda < 10) {
        // inversion method
        var expMean_1 = Math.exp(-lambda);
        return function () {
            var p = expMean_1;
            var x = 0;
            var u = random.next();
            while (u > p) {
                u = u - p;
                p = (lambda * p) / ++x;
            }
            return x;
        };
    }
    else {
        // generative method
        var smu_1 = Math.sqrt(lambda);
        var b_1 = 0.931 + 2.53 * smu_1;
        var a_1 = -0.059 + 0.02483 * b_1;
        var invAlpha_1 = 1.1239 + 1.1328 / (b_1 - 3.4);
        var vR_1 = 0.9277 - 3.6224 / (b_1 - 2);
        return function () {
            var _a;
            while (true) {
                var u = void 0;
                var v = random.next();
                if (v <= 0.86 * vR_1) {
                    u = v / vR_1 - 0.43;
                    return Math.floor(((2 * a_1) / (0.5 - Math.abs(u)) + b_1) * u + lambda + 0.445);
                }
                if (v >= vR_1) {
                    u = random.next() - 0.5;
                }
                else {
                    u = v / vR_1 - 0.93;
                    u = (u < 0 ? -0.5 : 0.5) - u;
                    v = random.next() * vR_1;
                }
                var us = 0.5 - Math.abs(u);
                if (us < 0.013 && v > us) {
                    continue;
                }
                var k = Math.floor(((2 * a_1) / us + b_1) * u + lambda + 0.445) | 0;
                v = (v * invAlpha_1) / (a_1 / (us * us) + b_1);
                if (k >= 10) {
                    var t = (k + 0.5) * Math.log(lambda / k) -
                        lambda -
                        logSqrt2PI +
                        k -
                        (1 / 12.0 - (1 / 360.0 - 1 / (1260.0 * k * k)) / (k * k)) / k;
                    if (Math.log(v * smu_1) <= t) {
                        return k;
                    }
                }
                else if (k >= 0) {
                    var f = (_a = logFactorial(k)) !== null && _a !== void 0 ? _a : 0;
                    if (Math.log(v) <= k * Math.log(lambda) - lambda - f) {
                        return k;
                    }
                }
            }
        };
    }
});
//# sourceMappingURL=poisson.js.map